/* This file is generated by glib-genmarshal, do not modify it. This code is licensed under the same license as the containing project. Note that it links to GLib, so must comply with the LGPL linking clauses. */
#pragma once

#include <glib-object.h>

G_BEGIN_DECLS

/* VOID:INT,INT (../at-spi2-core/atspi/atspimarshal.list:25) */
extern
void atspi_marshal_VOID__INT_INT (GClosure     *closure,
                                  GValue       *return_value,
                                  guint         n_param_values,
                                  const GValue *param_values,
                                  gpointer      invocation_hint,
                                  gpointer      marshal_data);

/* VOID:INT,STRING (../at-spi2-core/atspi/atspimarshal.list:26) */
extern
void atspi_marshal_VOID__INT_STRING (GClosure     *closure,
                                     GValue       *return_value,
                                     guint         n_param_values,
                                     const GValue *param_values,
                                     gpointer      invocation_hint,
                                     gpointer      marshal_data);

/* VOID:UINT,UINT,FLAGS,STRING (../at-spi2-core/atspi/atspimarshal.list:27) */
extern
void atspi_marshal_VOID__UINT_UINT_FLAGS_STRING (GClosure     *closure,
                                                 GValue       *return_value,
                                                 guint         n_param_values,
                                                 const GValue *param_values,
                                                 gpointer      invocation_hint,
                                                 gpointer      marshal_data);

/* VOID:OBJECT,INT,INT (../at-spi2-core/atspi/atspimarshal.list:28) */
extern
void atspi_marshal_VOID__OBJECT_INT_INT (GClosure     *closure,
                                         GValue       *return_value,
                                         guint         n_param_values,
                                         const GValue *param_values,
                                         gpointer      invocation_hint,
                                         gpointer      marshal_data);


G_END_DECLS
